"""
Act Book - Core automation framework for weboasis

Copyright 2024 Siyang Liu

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

from .core.registry import OperationRegistry
from .controller import ActBookController

# Only keep the decorator for operation registration
def register_operation(cls):
    """Decorator for easy operation registration."""
    return OperationRegistry.register(cls)

__all__ = [
    "register_operation",
    "ActBookController"
] 