"""
WebOasis - A web agent framework for researchers

A comprehensive framework for building and studying web agents for real-world applications.
Supports both automated and interactive (tutor-style) web agents with complex dynamic web page handling.

Copyright 2024 Siyang Liu

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

__version__ = "0.1.5"
__author__ = "Siyang Liu"
__email__ = "lsiyang@umich.edu"

# Import main components
from weboasis.ui_manager import SyncPlaywrightManager, SyncSeleniumManager

__all__ = [
    "SyncPlaywrightManager",
    "SyncSeleniumManager",
]
